/********************************************************************
Octavia Petrovici, May 2001

  SegGen.cpp

  Random segment generating code in C++. 

	To run: SegGen < file.in > file.out
	file.in has the number of desired segments, and file.out has the 
	coordinates of the generated segments, along with those
	of a randomly generated ray.
       
  This code is used to generate a random set of disjoint segments. 
  It generates pairs of random numbers in a given range and 
  it verifies if the corresonding segments intersect. 
	  
  The intersection code is described in "Computational Geometry in C" 
  (Second Edition), Author:Joseph O'Rourke, Chapter 7.  

	   
--------------------------------------------------------------------
This code is Copyright 2001 by Joseph O'Rourke.  It may be freely 
redistributed in its entirety provided that this copyright notice is 
not removed.
--------------------------------------------------------------------

*********************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <math.h>
#include <time.h>
#define	EXIT_FAILURE 1
#define	Xx 0
#define	Yy 1
#define MAX_VAL 500
#define MAX_TRYS 1000
#define	DIM 2                  /* Dimension of points */
typedef	int tPointi[DIM];      /* Type integer point */
typedef	double tPointd[DIM];   /* Type double point */

/*---------------------------------------------------------------------
							   Function prototypes.
---------------------------------------------------------------------*/

char	SegSegInt( tPointi a, tPointi b, tPointi c, tPointi d, tPointd p );
char	ParallelInt( tPointi a, tPointi b, tPointi c, tPointi d, tPointd p );
bool	Between( tPointi a, tPointi b, tPointi c );
void	Assigndi( tPointd p, tPointi a );
void	Assigni( tPointi p, tPointi a );
int		Collinear( tPointi a, tPointi b, tPointi c );
int     AreaSign( tPointi a, tPointi b, tPointi c );

/*-------------------------------------------------------------------*/

main(){

	tPointi a,b;				/* a and b are the temporary storage for the newly generated segments*/
	tPointi c[10000], d[10000]; /* c[] and d[] contain the coordinates of the accepted segment endpoints*/
	tPointd p;					/* intersection point*/
	char code;
	char ans;
	int NS;                     //desired number of segments
	int currNS = 0;
	int trys; 
	bool gotOne = false;
	int x = ((MAX_VAL/2)+(rand()%(MAX_VAL/2)))-(MAX_VAL/4);
	int y = ((MAX_VAL/2)+(rand()%(MAX_VAL/2)))-(MAX_VAL/4);

	printf("Desired number of segments?\n");
	scanf("%d", &NS);

	if (NS>10000){
	printf("Too big!");
	exit(0);	
	}

	printf("%d\n", NS);
	time_t t1;
	(void) time(&t1);
	srand((long) t1);

	//generate first set of random points determining a segment
	c[0][Xx] = rand()%MAX_VAL;
	c[0][Yy] = rand()%MAX_VAL;
	d[0][Xx] = rand()%MAX_VAL;
	d[0][Yy] = rand()%MAX_VAL;
	currNS ++;

	//output: 
	//printf("Generating segments.\n");
	printf("%d %d %d %d\n", c[0][Xx], c[0][Yy], d[0][Xx], d[0][Yy]);

	//generate the rest
	while (currNS <NS){
		trys = 0;
		gotOne = false;
		while ((trys < MAX_TRYS) && (!gotOne)){	
			trys++;
			//generate random segment
			a[0] = rand()%MAX_VAL;
			a[1] = rand()%MAX_VAL;
			b[0] = rand()%MAX_VAL;
			b[1] = rand()%MAX_VAL;
			gotOne = true;
			for (int i = 0; i<currNS; i++){
				code = SegSegInt(a,b, c[i], d[i], p);		
				if (code != '0')				
					gotOne = false; 			   	
			}//end for							
			if (gotOne)	{			
				Assigni(c[currNS], a);			
				Assigni(d[currNS], b);			
				currNS++;
				//	printf("Generating segments.\n");
				printf("%d %d %d %d\n", a[0], a[1], b[0], b[1]);
			}//end if		
		}//end while !gotOne
		if (!gotOne)
	    printf("Error: never found a segment with %d trys\n", trys);		
	}//end while (currNS < NS)

	//***********************************random ray:
	//printf("Ray:\n");

	printf("%d %d\n1\n%d %d\n", x, y, rand()%MAX_VAL, rand()%MAX_VAL);
	printf("Bounces:\n");
	printf("5\n");
}

/*---------------------------------------------------------------------

SegSegInt: Finds the point of intersection p between two closed
segments ab and cd.  Returns p and a char with the following meaning:
'e': The segments collinearly overlap, sharing a point.
'v': An endpoint (vertex) of one segment is on the other segment,
but 'e' doesn't hold.
'1': The segments intersect properly (i.e., they share a point and
neither 'v' nor 'e' holds).
'0': The segments do not intersect (i.e., they share no points).
Note that two collinear segments that share just one point, an endpoint
of each, returns 'e' rather than 'v' as one might expect.

---------------------------------------------------------------------*/

char	SegSegInt( tPointi a, tPointi b, tPointi c, tPointi d, tPointd p )
{
	double  s, t;       /* The two parameters of the parametric eqns. */
	double num, denom;  /* Numerator and denoninator of equations. */
	char code = '?';    /* Return char characterizing intersection. */
	
	denom = a[Xx] * (double)( d[Yy] - c[Yy] ) +
		b[Xx] * (double)( c[Yy] - d[Yy] ) +
		d[Xx] * (double)( b[Yy] - a[Yy] ) +
		c[Xx] * (double)( a[Yy] - b[Yy] );
	
	/* If denom is zero, then segments are parallel: handle separately. */
	if (denom == 0.0)
		return  ParallelInt(a, b, c, d, p);

	num =    a[Xx] * (double)( d[Yy] - c[Yy] ) +
		c[Xx] * (double)( a[Yy] - d[Yy] ) +
		d[Xx] * (double)( c[Yy] - a[Yy] );

	if ( (num == 0.0) || (num == denom) ) code = 'v';
	
	s = num / denom;
	
	num = -( a[Xx] * (double)( c[Yy] - b[Yy] ) +
		b[Xx] * (double)( a[Yy] - c[Yy] ) +
		c[Xx] * (double)( b[Yy] - a[Yy] ) );

	if ( (num == 0.0) || (num == denom) ) code = 'v';

	t = num / denom;
	
	if      ( (0.0 < s) && (s < 1.0) &&
		(0.0 < t) && (t < 1.0) )
		code = '1';
	else if ( (0.0 > s) || (s > 1.0) ||
		(0.0 > t) || (t > 1.0) )
		code = '0';
	
	p[Xx] = a[Xx] + s * ( b[Xx] - a[Xx] );
	p[Yy] = a[Yy] + s * ( b[Yy] - a[Yy] );
	
	return code;
}

char	ParallelInt( tPointi a, tPointi b, tPointi c, tPointi d, tPointd p )
{
	if ( !Collinear( a, b, c) ) 
		return '0';
	if ( Between( a, b, c ) ) {
		Assigndi( p, c );
		return 'e';
	}
	if ( Between( a, b, d ) ) {
		Assigndi( p, d );
		return 'e';
	}
	if ( Between( c, d, a ) ) {
		Assigndi( p, a );
		return 'e';
	}
	if ( Between( c, d, b ) ) {
		Assigndi( p, b );
		return 'e';
	}
	return '0';
}

void	Assigni( tPointi p, tPointi a )

{
	int i;
	for ( i = 0; i < DIM; i++ )		
	     p[i] = a[i];	
}

void	Assigndi( tPointd p, tPointi a )
{
	int i;
	for ( i = 0; i < DIM; i++ )
	      p[i] = a[i];
}


/*---------------------------------------------------------------------

Returns TRUE iff point c lies on the closed segement ab.
Assumes it is already known that abc are collinear.

---------------------------------------------------------------------*/

bool    Between( tPointi a, tPointi b, tPointi c )
{
	tPointi      ba, ca;
	
	/* If ab not vertical, check betweenness on x; else on y. */
	if ( a[Xx] != b[Xx] )
		return ((a[Xx] <= c[Xx]) && (c[Xx] <= b[Xx])) ||
		((a[Xx] >= c[Xx]) && (c[Xx] >= b[Xx]));
	else
		return ((a[Yy] <= c[Yy]) && (c[Yy] <= b[Yy])) ||
		((a[Yy] >= c[Yy]) && (c[Yy] >= b[Yy]));
}

int Collinear( tPointi a, tPointi b, tPointi c )
{
	return AreaSign( a, b, c ) == 0;
}

int     AreaSign( tPointi a, tPointi b, tPointi c )
{
    double area2;	
    area2 = ( b[0] - a[0] ) * (double)( c[1] - a[1] ) -
		( c[0] - a[0] ) * (double)( b[1] - a[1] );
	
    /* The area should be an integer. */
    if      ( area2 >  0.5 ) return  1;
    else if ( area2 < -0.5 ) return -1;
    else                     return  0;
}
